package alg;

import java.io.*;
import java.util.*;

public class FormatText {

static char EMPTYLINE = '\12';  // octal!!
static char CR = '\15';
static char LF = '\12';
static char SPACE = '\40';
static char NULL = '\00';


static int pagew;
static int pageh;
static char [][] page;

static int ncolumns;
static int nlines;
static int [] lineC; // capacity (max no of chars) of output line on the page
static int [] lineX; // X coord of the line
static int [] lineY; // Y coord of the line

static int iline;    // current output line index
static char align;

static char [] inText = new char[1];   // to be expanded during reading
static int inTextLen = 0;

static boolean isEndOfInputText() {
  if (inTextLen >= 6)   // at least 6 chars for "[Align" string;
    if ( (inText[inTextLen-6] == '[') && (inText[inTextLen-5] == 'A') &&
         (inText[inTextLen-4] == 'l') && (inText[inTextLen-3] == 'i') &&
         (inText[inTextLen-2] == 'g') && (inText[inTextLen-1] == 'n') )
      return true;
  return false;
}

static void addCharToInText(char c) {
  if (inTextLen == inText.length) {  // if array is full, extend it
    char [] newArr = new char[inText.length*2];
    for(int i = 0; i < inText.length; i++)
      newArr[i] = inText[i];
    inText = newArr;
  }
  inText[inTextLen++] = c;
}

static void readInputText(BufferedReader br ) throws IOException {
  int c;
  boolean wasSpace = false;
  boolean isEmptyLine = true;
  int noOfCharsOnLine = 0; //

   while (true) {
    c = br.read();

    // register line break  LF, (CR in Windows is just skipped)
    if ( c == (int) LF ) {
      if (isEmptyLine)
        addCharToInText(EMPTYLINE);
      wasSpace = !isEmptyLine; // line break serves as a separator = space
      isEmptyLine = true;
      noOfCharsOnLine = 0;
      continue;
    }

    // transfer a character which is a part of a word
    if (( (int) SPACE < c) && (c < 128)) {
      if ((c == (int) ']') && (noOfCharsOnLine == 6) && isEndOfInputText())
         break; // "[Align]" header reached, nothing more to read;
      if (wasSpace)
        addCharToInText(SPACE);
      addCharToInText((char) c);
      wasSpace = false;
      isEmptyLine = false;
      noOfCharsOnLine++;
      continue;
    }

    // register a space but do not transfer it to the text array
    if (c == (int) SPACE) {
       noOfCharsOnLine++;
      if (!isEmptyLine)
        wasSpace = true;
      continue;
    }
  }

  // adjust the end of the text correctly
  inTextLen -= 6;  // remove "[Align" substring,  ']' was not included
  while ((inTextLen > 0) &&
         ( (inText[inTextLen-1] == EMPTYLINE) || (inText[inTextLen-1] == SPACE)))
    inTextLen--;

  // skip to the next line of input file
  br.readLine();
}


static void readAndInitAll() throws IOException{
  // At the end of this function the character array inArr contains ONLY
  // the input words separated by SINGLE spaces OR (exclusively!)
  // by a sequence of empty lines  (=EMPTYLINE characters)
  BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
 
  // skip the [Page] header
  String line = br.readLine(); 
  
  // read the page parameters and init the page accordingly
  StringTokenizer st = new StringTokenizer(br.readLine());
  pageh = Integer.valueOf(st.nextToken());
  pagew = Integer.valueOf(st.nextToken());
  page = new char [pageh][pagew];
  ncolumns = Integer.valueOf(st.nextToken());
  nlines = ncolumns * (pageh-2);
  lineX = new int[nlines];
  lineY = new int[nlines];
  lineC = new int[nlines];
  int colx1, colx2;
  for(int j = 0; j < ncolumns; j++) {
    colx1 = Integer.valueOf(st.nextToken());
    colx2 = Integer.valueOf(st.nextToken());
     for (int y = j*(pageh-2); y < (j+1)*(pageh-2); y++){
      lineX[y] = colx1;
      lineY[y] = y % (pageh-2) + 1; //
      lineC[y] = colx2-colx1+1;
    }
  }
  String border = br.readLine().trim().substring(1, 9);
  pageBorder(page, border);


  // skip the [Pictures] header
  line = br.readLine();

  // read the Picture parameters
  st = new StringTokenizer(br.readLine());
  int imgno = Integer.valueOf(st.nextToken());
  int tlx, tly, brx, bry;
  String fill;
  for (int k = 0; k < imgno; k++) {
    st = new StringTokenizer(br.readLine());
    tly = Integer.valueOf(st.nextToken());
    tlx = Integer.valueOf(st.nextToken());
    bry = Integer.valueOf(st.nextToken());
    brx = Integer.valueOf(st.nextToken());
    fill = st.nextToken();
    // draw the picture directly on the page
    if (fill.length() == 1) fill = "\"" + " "+ "\"";
    for (int i = tly; i <= bry; i++)  
      for(int j = tlx; j <= brx; j++)
        page[i][j] = fill.charAt(1);

    // adjust x-coords or length of each line which collides with the picture
    for (int i = 0; i < nlines; i++) {  
      if ((tly <= lineY[i] ) && (lineY[i] <= bry)) {
        if ((lineX[i] < tlx) && (tlx < lineX[i]+ lineC[i]))
           lineC[i] = tlx-lineX[i];
        else
        if ((lineX[i] <= brx) && (brx < lineX[i]+ lineC[i]-1 ))
          {lineC[i] -= (brx-lineX[i]+1); lineX[i] = brx+1;    }
        else
        if (( tlx <= lineX[i]) && (lineX[i]+lineC[i] <= brx))
          lineC[i] = 0;
      }
  }
  } // end of reading pictures

  // now get rid of all lines of zero lengths which might have been erased
  // by some pictures == move lines of nonzero length forward in the array
  int i1 = 0;
  while ((i1 < nlines) && (lineC[i1] != 0)) i1++;
  if (i1 < nlines) {
    for (int i2 = i1+1; i2 < nlines; i2++)
      if (lineC[i2] != 0) {
       lineX[i1] = lineX[i2];
       lineY[i1] = lineY[i2];
       lineC[i1++] = lineC[i2];
      }
  }
  nlines = i1; // possibly reduced number of lines after erasing those with 0 length

  // skip the [Text] header
  line = br.readLine(); 

  // read the text itself into the inText array
  readInputText(br )  ;

  // read the align specification ([Align] header was read in the previous call)
  align = br.readLine().trim().charAt(0);

  } // end of readAndInitAll



static int jin0 = 0; // first char to appear on a current output line sits at this index in inText
static  int jin1 ;  // index of first char of current word in the inText
static  int jin2 ;  // index of current char


// after text analysis j2 points behind the text, thus anything at j2 is not transferred,
// nevertheless, the char at j2 influences what to do with the text

static int transfer() {
   // returns -1 if text ends within page
  // otherwise returns index of last char in inArr which falls into the output page
  // thus non-negative value returned indicates that text does not fit into the page

  // This method 'chops' the input text preprocessed in the readInputText
  // method into short pieces each of which fits one successive line at the
  // output page.


 jin2 = 0; // to start at 0
 jin1 = -2; // index of the last used character for current output line
 iline = 0; //
 while (true) {
   if (jin2 >= inTextLen)
     return -1; // end of input text reached
   if (iline >= nlines) //
      return jin1;
   if (inText[jin2] == EMPTYLINE) {
        iline++; jin0 = ++jin2; jin1 = jin0-1; // just shift all to new pos
     }
   else { // some unempty line is at the input we might be in the middle of it
     jin2 += (inText[jin2] == ' ')? 1 : 0; // skip the space if present
     while ((jin2 < inTextLen) &&  (inText[jin2] != ' ') && (inText[jin2] != EMPTYLINE))
       jin2++;  // find the end of the word
     if ((jin2 -jin0) < lineC[iline]-1) { // plenty of space at the out line
       jin1 = jin2-1;
       if ((jin2 == inTextLen) || (inText[jin2] == EMPTYLINE))
         putLine(jin1);
     }
     else // the current word between jin1 and jin2 may overflow or just fit the current output line
       if (((jin2 -jin0) == lineC[iline]-1) || ((jin2 -jin0) == lineC[iline])) { // just fits the line
         putLine(jin2-1);
       }
       else { // word begins at current line and overflows it
         if (iline == nlines -1) {  // no output line available for the rest of the word
           putLine(jin0+lineC[iline]-1);
           return jin1;
         }
         if ((jin0 <= jin1) && (lineC[iline+1] >= (jin2-jin1-2))){ // enough capac on the next line and current unempty
           putLine(jin1);  // put current line without the current word (postpone the word to the next leg)
         }
         else  // not enough capac on the next line or empty current line
           while (true) {  // chop the word into more pieces filling entire line(s)
             jin1 = Math.min(jin0+lineC[iline]-1, jin2-1);
             if ((1+jin1-jin0 >= lineC[iline] -1) || ((jin2 < inTextLen) && (inText[jin2] == EMPTYLINE)))
                putLine(jin1); // only if either full or followed by EOL.
             if (iline >= nlines)
               return jin1;  // no more space at the output page
             jin0 = jin1+1;
             if (jin0 == jin2)
               break; // whole word transferred, can move to the next one
           }
       }
   }
 }
}

static void putLine(int last) {
    int shift = 0;
  if (align  == 'r')
     shift = lineC[iline] - (last+1-jin0);
  if (align  == 'c')
     shift = (lineC[iline] - (last+1-jin0))/2;
  for(int j = 0; j <= last-jin0; j++)
    page[lineY[iline]][lineX[iline]+j+shift] = inText[jin0+j];
 iline++;
 jin1 = last;
 jin0 = jin2 = ((last < inTextLen-1))&&(inText[last+1] == ' ') ? last+2: last+1;
}

static void pageBorder(char [][] page, String borderChars) {
  page[0][0] = borderChars.charAt(0);
  page[0][pagew-1] = borderChars.charAt(2);
  page[pageh-1][0] = borderChars.charAt(5);
  page[pageh-1][pagew-1] = borderChars.charAt(7);
  for(int j = 1; j < pagew-1; j++) {
    page[0][j] = borderChars.charAt(1);
    page[pageh-1][j] = borderChars.charAt(6);
  }
  for(int i = 1; i < pageh-1; i++) {
    page[i][0] = borderChars.charAt(3);
    page[i][pagew-1] = borderChars.charAt(4);
  }
 for (int i = 1; i < pageh-1; i++)
   for(int j = 1; j < pagew-1; j++)
     page[i][j] = ' ';
}

static void listPage(char [][] page) {
  for (int i = 0; i < page.length; i++) {
    for(int j = 0; j < page[0].length; j++)
      System.out.printf("%s", page[i][j]);
    System.out.printf("\n" );
  }
}

  public static void main(String[] args) throws IOException {
    readAndInitAll();
    int failure = transfer();
    listPage(page);
    if (failure > -1)
      System.out.printf("[Some text missing]\n" );
  }


}
